<?php
/**
 * ------------------------------------------------------------------------
 * JA Filter Plugin - Docman
 * ------------------------------------------------------------------------
 * Copyright (C) 2004-2016 J.O.O.M Solutions Co., Ltd. All Rights Reserved.
 * @license - GNU/GPL, http://www.gnu.org/licenses/gpl.html
 * Author: J.O.O.M Solutions Co., Ltd
 * Websites: http://www.joomlart.com - http://www.joomlancers.com
 * This file may not be redistributed in whole or significant part.
 * ------------------------------------------------------------------------
 */
defined('_JEXEC') or die;

class DocmanFilterHelper extends BaseFilterHelper {
	
	function __construct($params = array())
	{
		$this->_db = JFactory::getDbo();
		return parent::__construct($params);
	}
	
	function getLangSuffix()
	{
		$langs = JFactory::getLanguage()->getKnownLanguages();
		$lang_sfx = array();
		foreach ($langs as $lang) {
			$lang_sfx[] = $lang['tag'];
		}
		return $lang_sfx;
	}
	
	function getFilterItems($catid)
	{
		$filterItems = array();
		$lang_sfx = $this->getLangSuffix();

		foreach ($lang_sfx AS $tag) {
			$filterItems[strtolower(str_replace('-','_',$tag))] = $this->getItemList($catid);
		}

		return $filterItems;
	}
	
	public function getItemList($catid)
	{
		$itemList = new stdCLass();

		if ($catid != '0') {
			$catsList = array($catid);
			foreach ($catsList as $cat) {
				$level = $this->getCategoryLevel($cat);
				if ($childs = $this->getChildCategories($cat, (int)$level)) {
					foreach ($childs as $child) {
						$catsList[] = $child;
					}
				}
			}
		} else {
			$catsList = array();
			if ($this->getDocmanTreeCategories()) {
				foreach ($this->getDocmanTreeCategories() as $cat) {
					$catsList[] = $cat->id;
				}
			}
		}

		$itemIdList = $this->getListId($catsList);
		if ($itemIdList) {
			foreach ($itemIdList as $id) {
				$property = 'item_'.$id;
				$item = $this->getItem($id, $catsList);
				if( !empty($item))
					$itemList->{ $property } = $item;
				else
					continue;
			}
		}
		
		return $itemList;
	}
	
	public function getItem($id, $catsList)
	{
		$baseItem = $this->getBaseItem($id);
		$itemParams = json_decode($baseItem->params);
		$desc = strip_tags($baseItem->desc);
		
		$item = new stdCLass();
		$item->show_info = $this->show_info ? true : false;
		$item->id = $id;
		$item->name = $baseItem->title;
		$item->thumbnail = !empty($baseItem->image) ? $baseItem->image : '';
		$item->icon = $itemParams->icon;
		$item->isVideo = ($item->icon == 'video') ? true : false;
		$item->desc = !empty($desc) ? utf8_encode(substr($desc,0,100)).'...' : utf8_encode($desc);
		$item->created_by = $baseItem->created_by;
		$item->access = $baseItem->access;
		
		$item->attr = array();
		if ($this->checkDisplayOnFO('name')) {
			$item->attr['name']['frontend_value'] = $item->name;
			$fieldconfig = $this->getFieldConfig('name');
			$item->attr['name']['title'] = array($fieldconfig['title']);
			$item->attr['name']['type'] = $fieldconfig['type'];
		}
		
		//Url and Url download.
		$alias = $id.'-'.$baseItem->slug;
		$category_slug = $this->getCategorySlug($id);
		$Itemid = $this->getItemid();
		$item->url = "index.php?option=com_docman&view=document&alias=$alias&category_slug=$category_slug&Itemid=$Itemid";
		$item->url_download = "index.php?option=com_docman&view=download&alias=$alias&category_slug=$category_slug&Itemid=$Itemid";
		$item->published_date = array (strtotime($baseItem->publish_on));
		
		// Downloads
		$item->download = ((int)$baseItem->hits < 2) ? ((int)$baseItem->hits == 1 ? 1 : 0) : 0 ;
		$item->downloads = (int)$baseItem->hits;
		
		if ($this->checkDisplayOnFO('downloads')) {
			$item->attr['downloads']['frontend_value'] = $item->downloads ? $item->downloads : $item->download;
			$fieldconfig = $this->getFieldConfig('downloads');
			$item->attr['downloads']['title'] = array($fieldconfig['title']);
			$item->attr['downloads']['type'] = $fieldconfig['type'];
		}
		
		if ($baseItem->publish_on != '0000-00-00 00:00:00') {
			if ($this->checkPublished('published_date')) {
				$item->published_date = array (strtotime($baseItem->publish_on));
			}

			if ($this->checkDisplayOnFO('published_date')) {
				$item->attr['published_date']['frontend_value'] = array (strtotime($baseItem->publish_on));
				$fieldconfig = $this->getFieldConfig('published_date');
				$item->attr['published_date']['title'] = array($fieldconfig['title']);
				$item->attr['published_date']['type'] = $fieldconfig['type'];
			}
		}
		
		if ($baseItem->created_on != '0000-00-00 00:00:00') {
			if ($this->checkPublished('created_date')) {
				$item->created_date = array( strtotime($baseItem->created_on) );
			}

			if ($this->checkDisplayOnFO('created_date')) {
				$item->attr['created_date']['frontend_value'] = array( strtotime($baseItem->created_on) );
				$fieldconfig = $this->getFieldConfig('created_date');
				$item->attr['created_date']['title'] = array($fieldconfig['title']);
				$item->attr['created_date']['type'] = $fieldconfig['type'];
			}
		}

		if ($baseItem->modified_on != '0000-00-00 00:00:00') {
			if ($this->checkPublished('modified_date')) {
				$item->modified_date = array( strtotime($baseItem->modified_on) );
			}

			if ($this->checkDisplayOnFO('modified_date')) {
				$item->attr['modified_date']['frontend_value'] = array( strtotime($baseItem->modified_on) );
				$fieldconfig = $this->getFieldConfig('modified_date');
				$item->attr['modified_date']['title'] = array($fieldconfig['title']);
				$item->attr['modified_date']['type'] = $fieldconfig['type'];
			}
		}
		
		//Attributes
		$this->attr = array();
		//Category Info
		$this->getCategoryInfo($id, $catsList);
		//Tag Info
		$this->getTagInfo($id);
		//File Type
		$this->getFileType($item->icon);
		
		$item->attr = array_merge($item->attr, $this->attr);
		
		return $item;
	}
	
	public function getBaseItem($id)
	{
		$query = $this->_db->getQuery(true);
		$query->select('dd.*')
			->from('#__docman_documents as dd')
			->where('dd.docman_document_id = ' . (int) $id);
		$this->_db->setQuery($query);
		$baseItem = $this->_db->loadObject();
		
		return $baseItem;
	}
	
	public function getListId($catids)
	{
		$query = $this->_db->getQuery(true);
		$query ->select('dd.docman_document_id as id')->from('#__docman_documents as dd')
				->where('dd.docman_category_id IN ('.implode(',', $catids).') AND dd.`enabled` = "1"')
				->order('dd.title ASC');
		$this->_db->setQuery($query);
		$listId = $this->_db->loadColumn();
		
		return $listId;
	}
	
	public function getCategorySlug($document_id)
	{
		$query = $this->_db->getQuery(true);
		$query->select('dc.slug')
				->from('#__docman_categories as dc')
				->join('LEFT', '#__docman_documents as dd ON dd.docman_category_id = dc.docman_category_id')
				->where('dd.docman_document_id = '. (int) $document_id);
		$this->_db->setQuery($query);
		$slug = $this->_db->loadResult();
		
		return $slug;
	}
	
	public function getItemid()
	{
		$query = $this->_db->getQuery(true);
		$query->select('id')
				->from('#__menu')
				->where('`link` REGEXP "^(index.php.*com_docman)(&view=list|tree|document)"');
		$this->_db->setQuery($query);
		
		return $this->_db->loadResult();
	}
	
	public function getFileType($icon)
	{
		if ($icon == 'default') {
			$icon = 'ALTER_TEXT_FOR_DEAULT';
		}
		
		if ($this->checkPublished('attr.type.value')) {
			$this->attr['type']['value'] = array($icon);
			$this->attr['type']['frontend_value'] = array('<span class="k-icon-document-'.$icon.'"></span>'.' '.strtoupper($icon));
		}

		if ( $this->checkDisplayOnFO('attr.type.value')) {
			$this->attr['type']['frontend_value'] = array('<span class="k-icon-document-'.$icon.'"></span>'.' '.strtoupper($icon));
			$fieldconfig = $this->getFieldConfig('attr.type.value');
			$this->attr['type']['title'] = array($fieldconfig['title']);
			$this->attr['type']['type'] = $fieldconfig['type'];
		}
		
		return $this->attr;
	}
	
	public function getCategoryInfo($id, $catsList)
	{
		$query = $this->_db->getQuery(true);
		$query->select ('dc.docman_category_id as id, dc.title as name')
				->from('#__docman_categories as dc')
				->join('LEFT', '#__docman_documents as dd ON dd.docman_category_id = dc.docman_category_id')
				->where('dd.docman_document_id = '. (int) $id);
		$this->_db->setQuery($query);
		$category = $this->_db->loadObject();

		if ($category) {
			$parentTree = $this->getParentList($category->id);
			$nameTree = $category->name;
			$parenName = '';
			if ($parentTree) {
				foreach ($parentTree as $parent) {
					$parenName .= $parent->name . ' &raquo; ';
				}
			}
			
			if ($this->checkPublished('attr.cat.value')) {
				$this->attr['cat']['value'][] = $category->id;
				$this->attr['cat']['frontend_value'] = array( $parenName . $nameTree );
			}
			
			if ( $this->checkDisplayOnFO('attr.cat.value')) {
				$this->attr['cat']['frontend_value']= array( $parenName . $nameTree );
				$fieldconfig = $this->getFieldConfig('attr.cat.value');
				$this->attr['cat']['title'] = array($fieldconfig['title']);
				$this->attr['cat']['type'] = $fieldconfig['type'];
			}
		}
		
		return $this->attr;
	}
	
	public function getParentList($catid)
	{
		$query = $this->_db->getQuery(true);
		$query->select('dc.docman_category_id as id, dc.title as name')
				->from('#__docman_category_relations as dcr')
				->join('LEFT', '#__docman_categories as dc ON dc.docman_category_id = dcr.ancestor_id')
				->where('dcr.descendant_id = ' . (int) $catid . ' AND dcr.descendant_id != dcr.ancestor_id')
				->order('id AND dcr.`level`');
		$this->_db->setQuery($query);
		$parentCats = $this->_db->loadObjectList();
		
		return $parentCats;
	}
	
	public function getTagInfo($id) 
	{
		$query = $this->_db->getQuery(true);
		$query->select('dt.tag_id as id, dt.title as tag_name')
				->from('#__docman_tags_relations as dtr')
				->join('LEFT', '#__docman_tags as dt ON dt.tag_id = dtr.tag_id')
				->join('LEFT', '#__docman_documents as dd ON dd.uuid = dtr.row')
				->where('dd.docman_document_id = ' . (int) $id);
		$this->_db->setQuery($query);
		$tags = $this->_db->loadObjectList();
		
		if ($tags) {
			$tdata = array();
			foreach ($tags as $tag) {
				$tdata['value'][] = $tag->id;
				$tdata['frontend_value'][] = $tag->tag_name;
			}
			
			if ($this->checkPublished('attr.tag.value')) {
				$this->attr['tag'] = $tdata;
			}

			if ( $this->checkDisplayOnFO('attr.tag.value')) {
				$this->attr['tag']['frontend_value'] = $tdata['frontend_value'];
				$fieldconfig = $this->getFieldConfig('attr.tag.value');
				$this->attr['tag']['title'] = array($fieldconfig['title']);
				$this->attr['tag']['type'] = $fieldconfig['type'];
			}
		}
		
		return $this->attr;
	}
	
	public function getCategoryLevel($catid)
	{
		$query = $this->_db->getQuery(true);
		$query->select('(count(drc.level) - 1) as level')
			->from('#__docman_category_relations as drc')
			->where('descendant_id = ' . (int) $catid);
		$this->_db->setQuery($query);
		$level = $this->_db->loadResult();
		
		return $level;
	}
	
	public function getDocmanTreeCategories() 
	{
		$items = array();
		$query = $this->_db->getQuery(true);
		$query->select('dc.docman_category_id as id, dc.title as title, count(dcr.descendant_id) - 1 as level')
			->from('#__docman_categories as dc')
			->join('INNER', '#__docman_category_relations as dcr ON dc.docman_category_id = dcr.descendant_id')
			->GROUP('dc.docman_category_id')
			->ORDER('dc.title');
		$this->_db->setQuery($query);
		$categories = $this->_db->loadObjectList();
		
		if (count($categories)) {
			foreach ($categories as $cat) {
				if ($cat->level == '0') {
					$items[] = $cat;
					$childs = $this->getChildCategories($cat->id, (int)$cat->level);
					if ($childs) {
						foreach ($childs as $child) {
							foreach ($categories as $cat) {
								if ($child == $cat->id) {
									$items[] = $cat;
								}
							}
						}
					}
				}
			}
		}
		
		return $items;
	}
	
	public function getChildCategories($catid, $level) {
		if ($level == 0)
			$level = 1;
		$query = $this->_db->getQuery(true);
		$query ->select('dc.docman_category_id as id, drc.`level` as `level`')
			->from('#__docman_categories as dc')
			->join('INNER', '#__docman_category_relations as drc ON dc.docman_category_id = drc.descendant_id')
			//->join('INNER', '#__docman_documents as dd ON dd.docman_category_id = dc.docman_category_id')
			->where('drc.ancestor_id = '.(int) $catid . ' AND '.$this->_db->quoteName('drc.level').' = '. $level)
			//->group('dd.docman_category_id')
			->order('dc.title ASC');
		$this->_db->setQuery($query);
		$cats = $this->_db->loadObjectList();
		$items = array();
		
		if ($cats) {
			foreach ($cats as $cat) {
				$items[] = $cat->id;
				if ($this->getChildCategories($cat->id, (int)$cat->level)) {
					foreach ($this->getChildCategories($cat->id, (int)$cat->level) as $child) {
						$items[] = $child;
					}
				}
			}
		}
		
		return $items;
	}
}
